/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_COPLANAR_TASK_H
#define HPP_LOS_COPLANAR_TASK_H

#include "hpp/localstepper/task/task.hh"
class ChppLosRobotLink;
/**
A task to make a body line segment, defined by the coordinates of two points in the local frame of a joint, coplanar with a line given in worldframe
\ingroup tasks
 */
class ChppLosCoplanarTask: public ChppLosTaskOnJoint
{
    public:

        /**
        \brief Constructor
         */
        ChppLosCoplanarTask ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, CjrlJoint& inJoint, const vector3d& inLocalC, const vector3d& inLocalD, const vector3d& inWorldA,  const vector3d& inWorldNormal);
        /**
        \brief Set reference joint
         */
        void joint ( CjrlJoint& inJoint );
        void  localC ( const vector3d& inPoint ) ;
        void  localD ( const vector3d& inPoint ) ;
        void  worldA ( const vector3d& inPoint ) ;
        void  worldNormal ( const vector3d& inPoint ) ;
        const vector3d& localC() ;
        const vector3d& localD() ;
        const vector3d& worldA() ;
        const vector3d& worldNormal() ;

        /**
        \brief Set the gain for differential equation when solving this tasks
         */
        void maxVelocityNorm ( double inMV );
        /**
        \brief Return true
         */
        bool isEqualityTask() const {return true;};
        /**
        \brief Inherited
         */
        void computeDifferentialSystem();
        /**
        \brief Inherited
         */
        const vectorN& currentValue();
        

    protected:
        bool resize();
        void equivAsymMat3(const vector3d& inVec, matrixNxP& outMat);
        void crossProd3(const vector3d& v1, const vector3d& v2, vector3d& retV);
        vector3d attA,attC,attD,attNormal;
        ChppLosRobotLink* attRobotLink;
        double attMaxVelocity;
        matrixNxP attTempJacobian;
        matrix4d m;
        matrixNxP attMatNormal;
};

#endif

