/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/
#ifndef HPP_LOS_PARALLEL_TASK_H
#define HPP_LOS_PARALLEL_TASK_H

#include "hpp/localstepper/task/task.hh"
class ChppLosRobotLink;
/**
A task to align a vector in a robot's joint frame on a point in world frame
\ingroup tasks
 */
class ChppLosParallelTask: public ChppLosTaskOnJoint
{
    public:

        /**
        \brief Constructor
         */
        ChppLosParallelTask ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, CjrlJoint& inJoint, const vector3d& inLocalVector,  const vector3d& inTargetVector );
         /**
        \brief Set reference joint
          */
        void joint ( CjrlJoint& inJoint );
        /**
        \brief Set the local vector to be constrained
         */
        virtual void  localVector ( const vector3d& inV ) ;
        /**
        \brief Get the origin of the pointing vector.
         */
        virtual const vector3d& localVector() ;
        /**
        \brief Set the target vector in world coordinates
         */
        void  targetVector ( const vector3d& inV );
        /**
        \brief Get the target vector in world coordinates
         */
        const vector3d& targetVector();
        /**
        \brief Set the gain for differential equation when solving this tasks
         */
        void maxVelocityNorm ( double inMV );
        /**
        \brief Return true
         */
        bool isEqualityTask() const {return true;};
        /**
        \brief Inherited
         */
        void computeDifferentialSystem();
        /**
        \brief Inherited
         */
        const vectorN& currentValue();
        

    protected:
        bool resize();
        void Rodrigues(const vector3d& inW, double inAngle, matrixNxP& outRot);
        void equivAsymMat3(const vector3d& inVec, matrixNxP& outMat);
        void crossProd3(const vector3d& v1, const vector3d& v2, vector3d& retV);
        ChppLosRobotLink* attRobotLink;
        vector3d attLocalVector,attTargetVector;
        double attMaxVelocity;
        matrixNxP attTempJacobian;
        matrix4d m;
        matrixNxP matOP, matOT,matOTOP;
        matrix3d m3;
};

#endif
