/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/
#ifndef HPP_LOS_PLANE_TASK_H
#define HPP_LOS_PLANE_TASK_H

#include "hpp/localstepper/task/task.hh"
class ChppLosRobotLink;
/**
A 3D position task for a point located in the local frame of a given joint inside a CjrlDynamicHumanoidRobot inside a ChppLosCentipede object.
\ingroup tasks
 */
class ChppLosPlaneTask: public ChppLosTaskOnJoint
{
    public:

        /**
        \brief Constructor
         */
        ChppLosPlaneTask ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, CjrlJoint& inJoint, const vector3d& inLocalPoint, const vector3d& inTargetPlanePoint, const vector3d& inTargetPlaneNormal );
        /**
        \brief Get reference point in reference joint
         */
        const vector3d& localPoint();
        /**
        \brief Get the coordinates of a point in the target plane
         */
        const vector3d& targetPlanePoint();
        /**
        \brief Set reference point in reference joint local coordinates
         */
        void localPoint ( const vector3d& inVec );
        /**
        \brief Set a point in the target plane
         */
        void targetPlanePoint ( const vector3d& inVec );
        /**
        \brief Get the target plane's normal vector in world coordinates
         */
        const vector3d& targetPlaneNormal();
        /**
        \brief Set the target plane's normal vector in world coordinates
         */
        void targetPlaneNormal(const vector3d& inV);
        /**
        \brief Set the maximum linear velocity of the effector point
         */
        void maxVelocityNorm ( double inMV );
        /**
        \brief Return true
         */
        bool isEqualityTask() const {return attIsEquality;};
        /**
        \brief Return true
         */
        void isEqualityTask(bool inChoice) {attIsEquality = inChoice;};
        /**
        \brief Inherited
         */
        void computeDifferentialSystem();
        /**
        \brief Inherited
         */
        const vectorN& currentValue();

    private:
        bool resize();
        vector3d attCurrentPoint;
        ChppLosRobotLink* attRobotLink;
        vector3d attLocalPoint,attPlanePoint, attPlaneNormal, tempV;
        double attMaxVelocity;
        matrixNxP attTempJacobian, attPlaneNormalM;
        matrix4d m;
        bool attIsEquality;
};

#endif
