/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_SOLVER_BASIC_H
#define HPP_LOS_SOLVER_BASIC_H

#include "MatrixAbstractLayer/MatrixAbstractLayer.h"
class ChppHikSolverBasic;
class ChppLosTask;
class ChppLosCentipede;


/**
\brief This object solves a stack of prioritized equality and inequality tasks applied to a ChppLosCentipede object.
\ingroup solvers
 */
class ChppLosSolverBasic
{
    public:

        /**
        \brief Constructor.
         */
        ChppLosSolverBasic ( ChppLosCentipede& inCentipede );
        /**
        \brief Set the constraints of the problem.
         */
        void constraints ( const std::vector<ChppLosTask*>& inConstraints );

        /**
        \brief Set the prioritized tasks to be solved. First task in the vector has highest priority.
         */
        void tasks ( const std::vector<ChppLosTask*>& inTasks );

        /**
        \brief Set the joints used in solving. Size of vector must be equal to Centipede's number of dofs
         */
        bool setMask ( const vectorN& inActiveDofs );

        /**\brief Solve
         */
        bool solve ( vectorN& io_guess );

        /**
        \brief Destructor
         */
        ~ChppLosSolverBasic();

    private:

        void loadConstraintsInSolver();
        void resize();
        ChppLosCentipede* attCentipede;
        ChppHikSolverBasic* attHikSolver;
        std::vector<ChppLosTask*> attTasks,attConstraints;
        vectorN attActivatedJoints, attSolution, attUb,attLb;
};

#endif
