/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_TASK_H
#define HPP_LOS_TASK_H

#include "MatrixAbstractLayer/MatrixAbstractLayer.h"
class ChppHikLinearSystem;
class ChppLosCentipede;
class CjrlJoint;
/**
Abstract interface for a task. A task produces a linear system that can be composed either of equalities or inequalities. The Linear system is dependant on the state of the ChppLosCentipede object passed to the task's constructor. The linear system is updated through call of method computeDifferentialSystem() and can be retrieved through call of method differentialSystem().
\ingroup tasks
 */
class ChppLosTask
{
    public:
        /**
        \brief Constructor
         */
        ChppLosTask(ChppLosCentipede& inCentipede);
        /**
        \brief Destructor
         */
        virtual ~ChppLosTask();
        /**
        \brief Compute the linear system to be solved for this task at the current state
         */
        virtual void computeDifferentialSystem() = 0;
        /**
        \brief Return the value of the task. The task is strictly solved if the norm of the returned vector is null.
         */
        virtual const vectorN& currentValue() = 0;
        /**
        \brief Tell if this task is an equality task
         */
        virtual bool isEqualityTask() const = 0;
        /**
        \brief Get the linear system to be solved for this task at the current state
         */
        virtual ChppHikLinearSystem& differentialSystem();
        /**
        \brief Get the damping factor associated with the linear system
         */
        virtual double dampingFactor();
        /**
        \brief Set the damping factor associated with the linear system
         */
        virtual void dampingFactor ( double inF );
        /**
        \brief tag with a start time
         */
        void startTime(double inT);
        /**
        \brief get the start time (return 0 if not set)
         */
        double startTime();
        /**
        \brief tag with an end time
         */
        void endTime(double inT);
        /**
        \brief get the end time (return 0 if not set)
         */
        double endTime();

    protected:
        virtual bool resize();
        ChppLosCentipede* attCentipede;
        ChppHikLinearSystem* attConstraint;
        double attDamp,attStartTime,attEndTime;
        vectorN attValue;
};

/**
A task that uses a joint in the robot link
\ingroup tasks
 */
class ChppLosTaskOnJoint: public ChppLosTask
{
    public:
        ChppLosTaskOnJoint(ChppLosCentipede& inCentipede,CjrlJoint& inJoint);
        /**
        \brief Get reference joint
         */
        CjrlJoint* joint();
        /**
        \brief Get reference joint
         */
        void joint(CjrlJoint* inJoint);
        
    protected:
        CjrlJoint* attJoint;
};

#endif
