#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpp/localstepper/centipede.hh"
#include "hpp/localstepper/link.hh"

#define M4_IJ MAL_S4x4_MATRIX_ACCESS_I_J

ChppLosCentipede::ChppLosCentipede ( const ChppLosFootprint& initialleftprint, const ChppLosFootprint& initialrightprint )
{
    attInitialLeft = 0;
    attInitialRight = 0;
    initialLeftFootprint ( initialleftprint );
    initialRightFootprint ( initialrightprint );
    attNumberDof = 0;
}

ChppLosCentipede::~ChppLosCentipede()
{
    delete attInitialLeft;
    delete attInitialRight;
}

const ChppLosFootprint& ChppLosCentipede::initialLeftFootprint()
{
    return *attInitialLeft;
}

const ChppLosFootprint& ChppLosCentipede::initialRightFootprint()
{
    return *attInitialRight;
}

void ChppLosCentipede::initialLeftFootprint ( const ChppLosFootprint& initialleftprint )
{
    delete attInitialLeft;
    attInitialLeft = new ChppLosFootprint( initialleftprint );
    attInitialLeft->UXx = cos ( attInitialLeft->th );
    attInitialLeft->UXy = sin ( attInitialLeft->th );
    attInitialLeft->UYx = -attInitialLeft->UXy;
    attInitialLeft->UYy = attInitialLeft->UXx;
}

void ChppLosCentipede::initialRightFootprint ( const ChppLosFootprint& initialrightprint )
{
    delete attInitialRight;
    attInitialRight = new ChppLosFootprint(initialrightprint);
    attInitialRight->UXx = cos ( attInitialRight->th );
    attInitialRight->UXy = sin ( attInitialRight->th );
    attInitialRight->UYx = -attInitialRight->UXy;
    attInitialRight->UYy = attInitialRight->UXx;
}

void ChppLosCentipede::pop_back ( )
{
    if ( !attLinks.empty() )
    {
        std::list<ChppLosLink*>::iterator iter = attLinks.end();
        iter--;


        ChppLosSupportPolygonLink* spLink= dynamic_cast<ChppLosSupportPolygonLink*> ( *iter );

        if ( spLink )
            attSupportPolygonLinks.pop_back ( );
        else
            attRobotLinks.pop_back ( );

        attNumberDof -= ( *iter )->numberDof();

        attConfiguration.resize ( attNumberDof );
        attUb.resize ( attNumberDof );
        attLb.resize ( attNumberDof );

        attLinks.pop_back();
    }
}

void ChppLosCentipede::pop_front ( )
{
    if ( !attLinks.empty() )
    {
        std::list<ChppLosLink*>::iterator iter = attLinks.begin();

        ChppLosSupportPolygonLink* spLink= dynamic_cast<ChppLosSupportPolygonLink*> ( *iter );

        if ( spLink )
            attSupportPolygonLinks.pop_front ( );
        else
            attRobotLinks.pop_front ( );

        attNumberDof -= ( *iter )->numberDof();

        attConfiguration.resize ( attNumberDof );
        attUb.resize ( attNumberDof );
        attLb.resize ( attNumberDof );

        attLinks.pop_front();

        if ( !attLinks.empty() )
            ( *attLinks.begin() )->parent ( 0 );

        unsigned int rank = 0;
        for ( iter= attLinks.begin();iter!=attLinks.end();iter++ )
        {
            ( *iter )->rank ( rank );
            rank+= ( *iter )->numberDof();
        }
    }
}

bool ChppLosCentipede::push_front ( ChppLosLink& inLink )
{
    ChppLosSupportPolygonLink* spLink= dynamic_cast<ChppLosSupportPolygonLink*> ( &inLink );
    ChppLosRobotLink* rLink= dynamic_cast<ChppLosRobotLink*> ( &inLink );

    if ( spLink || rLink )
        attLinks.push_front ( &inLink );
    else
    {
        std::cout << "ChppLosCentipede::push_front -Error unrecognized type of link" <<std::endl;
        return false;
    }

    inLink.rank (0);
    attNumberDof += inLink.numberDof();

    if ( spLink )
        attSupportPolygonLinks.push_front ( spLink );
    else
        attRobotLinks.push_front ( rLink );

    attConfiguration.resize ( attNumberDof );
    attUb.resize ( attNumberDof );
    attLb.resize ( attNumberDof );


    if ( numberLinks() == 1 )
        inLink.parent ( 0 );
    else
    {
        std::list<ChppLosLink*>::iterator iter = attLinks.begin();
        iter++;
        ( *iter )->parent ( &inLink );
    }

    unsigned int rank = 0;
    std::list<ChppLosLink*>::iterator iter;
    for ( iter= attLinks.begin();iter!=attLinks.end();iter++ )
    {
        ( *iter )->rank ( rank );
        rank+= ( *iter )->numberDof();
    }
    return true;
}


bool ChppLosCentipede::push_back ( ChppLosLink& inLink )
{

    ChppLosSupportPolygonLink* spLink= dynamic_cast<ChppLosSupportPolygonLink*> ( &inLink );
    ChppLosRobotLink* rLink= dynamic_cast<ChppLosRobotLink*> ( &inLink );

    if ( spLink || rLink )
        attLinks.push_back ( &inLink );
    else
    {
        std::cout << "ChppLosCentipede::push_back -Error unrecognized type of link" <<std::endl;
        return false;
    }

    inLink.rank ( attNumberDof );
    attNumberDof += inLink.numberDof();

    if ( spLink )
        attSupportPolygonLinks.push_back ( spLink );
    else
        attRobotLinks.push_back ( rLink );

    attConfiguration.resize ( attNumberDof );
    attUb.resize ( attNumberDof );
    attLb.resize ( attNumberDof );

    if ( numberLinks() == 1 )
        inLink.parent ( 0 );
    else
    {
        std::list<ChppLosLink*>::iterator iter = attLinks.end();
        iter--;
        iter--;
        inLink.parent ( *iter );
    }

    return true;
}

void ChppLosCentipede::clearLinks()
{
    std::list<ChppLosLink*>::iterator iter ;
    for ( iter = attLinks.begin();iter!=attLinks.end();iter++ )
        ( *iter )->parent ( 0 );
    attLinks.clear();
    attSupportPolygonLinks.clear();
    attRobotLinks.clear();
    attNumberDof = 0;
    attConfiguration.resize ( 0 );
    attUb.resize ( 0 );
    attLb.resize ( 0 );

}

unsigned int ChppLosCentipede::numberDof()
{
    return attNumberDof;
}

unsigned int ChppLosCentipede::numberLinks()
{
    return attLinks.size();
}


ChppLosLink* ChppLosCentipede::linkAtRank ( unsigned int inRank )
{
    if ( inRank>=attLinks.size() )
    {
        std::cout << "ChppLosCentipede::linkAtRank -Error: index out of bounds"<< std::endl;
        return 0;
    }
    std::list<ChppLosLink*>::iterator iter = attLinks.begin();

    for ( unsigned int i=0;i<inRank;i++ )
    iter++;

    return ( *iter );
}


bool ChppLosCentipede::applyConfiguration ( const vectorN& inVector, bool computeRelativeFeetJacobians )
{
    if ( inVector.size() != attNumberDof )
    {
        std::cout << "ChppLosCentipede::applyConfiguration -Error: bad vector size"<< std::endl;
        return false;
    }

    const ChppLosFootprint* prevLeft = attInitialLeft;
    const ChppLosFootprint* prevRight = attInitialRight;
    std::list<ChppLosLink*>::iterator iter;

    for ( iter=attLinks.begin();iter!=attLinks.end();iter++ )
    {
        ( *iter )->applyConfiguration ( *prevLeft, *prevRight, inVector,computeRelativeFeetJacobians );

        prevLeft = & ( ( *iter )->leftprint() );
        prevRight = & ( ( *iter )->rightprint() );
    }

    return true;
}

bool ChppLosCentipede::getConfiguration ( vectorN& outVec )
{
    if ( outVec.size() != attNumberDof )
    {
        std::cout << "ChppLosCentipede::getConfiguration bad vector size" << std::endl;
        return false;
    }

    std::list<ChppLosLink*>::iterator iter ;
    for ( iter= attLinks.begin();iter!=attLinks.end();iter++ )
        ( *iter )->getConfiguration ( outVec );

    return true;
}

bool ChppLosCentipede::getUpperVelocityBounds ( vectorN& outVec )
{
    if ( outVec.size() != attNumberDof )
    {
        std::cout << "ChppLosCentipede::getUpperVelocityBounds bad vector size" << std::endl;
        return false;
    }

    std::list<ChppLosLink*>::iterator iter;
    for ( iter = attLinks.begin();iter!=attLinks.end();iter++ )
        ( *iter )->getUpperVelocityBounds ( outVec );

    return true;
}

bool ChppLosCentipede::getLowerVelocityBounds ( vectorN& outVec )
{
    if ( outVec.size() != attNumberDof )
    {
        std::cout << "ChppLosCentipede::getLowerVelocityBounds bad vector size" << std::endl;
        return false;
    }

    std::list<ChppLosLink*>::iterator iter ;
    for ( iter= attLinks.begin();iter!=attLinks.end();iter++ )
        ( *iter )->getLowerVelocityBounds ( outVec );

    return true;
}

bool ChppLosCentipede::getJacobian ( unsigned int startLinkRank,unsigned int effectorLinkRank, const vector3d& inAbsPoint, matrixNxP& outJacobian )
{
    if ( ( outJacobian.size2() != attNumberDof ) && ( outJacobian.size1() != 6 ) )
    {
        std::cout << "ChppLosCentipede::getJacobian -Error bad outJacobian size"<<std::endl;
        return false;
    }
    if ( ( startLinkRank >= numberLinks() ) || ( effectorLinkRank >= numberLinks() ) )
    {
        std::cout << "ChppLosCentipede::getJacobian -Error at least one link rank out of bounds"<<std::endl;
        return false;
    }
    if ( startLinkRank > effectorLinkRank )
    {
        std::cout << "ChppLosCentipede::getJacobian -Error startLinkRank should less or equal to effectorLinkRank"<<std::endl;
        return false;
    }
    outJacobian.clear();
    std::list<ChppLosLink*>::iterator iter=attLinks.begin();
    for ( unsigned int i=0;i<startLinkRank;i++ );
    iter++;
    for ( unsigned int i=startLinkRank;i<=effectorLinkRank;i++ )
    {
        ( *iter )->getJacobian ( inAbsPoint, outJacobian, ( *iter )->rank() );
        iter++;
    }

    return true;
}
