/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_MOTION_GENERATOR_H
#define HPP_LOS_MOTION_GENERATOR_H



#include <iostream>
#include "MatrixAbstractLayer/MatrixAbstractLayer.h"
#include <boost/numeric/bindings/traits/ublas_matrix.hpp>
#include "dynamicsJRLJapan/dynamicsJRLJapanFactory.h"
#include "hrp2Dynamics/hrp2OptHumanoidDynamicRobot.h"

#include "hpp/gik/robot/robot-motion.hh"
#include "hpp/localstepper/centipede.hh"
#include "hpp/localstepper/task/task.hh"
#include "hpp/localstepper/task/com-bounding-task.hh"
#include "hpp/localstepper/task/com-task.hh"
#include "hpp/localstepper/task/feet-projection-task.hh"
#include "hpp/localstepper/task/feet-anti-collisiontask.hh"
#include "hpp/localstepper/task/feet-area-task.hh"
#include "hpp/localstepper/task/disc-avoider.hh"
#include "hpp/localstepper/task/position-task.hh"
#include "hpp/localstepper/task/parallel-task.hh"
#include "hpp/localstepper/task/transformation-task.hh"
#include "hpp/localstepper/task/link-configuration-task.hh"
#include "hpp/localstepper/task/gaze-task.hh"
#include "hpp/localstepper/solver.hh"
#include "hpp/localstepper/centipede-factory.hh"
#include "hpp/localstepper/solver-basic.hh"
#include "hpp/localstepper/link.hh"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

class ChppLosMotionGenerator
{
    public:

        ChppLosMotionGenerator(const std::string& inPath);

        ~ChppLosMotionGenerator();

        void constraints(std::vector<ChppLosTask*>& constraints);
        void tasks(std::vector<ChppLosTask*>& tasks);
        
        void generate(ChppLosCentipede& inCentipede, const vectorN initialConfig);
        void generate(const vectorN initialConfig );
        ChppRobotMotion& motion();
        ChppLosCentipede& centipede(){return *attCentipede;}
        ChppLosRobotLink& robotLink(){return **attCentipede->robotLinks().begin();}
        
        bool setSolverMask(const vectorN& inVec);
        
        ChppGikStandingRobot& standingRobot(){return *attStandingRobot;}
        CjrlHumanoidDynamicRobot& robot(){return *attRobot;}
    
    private:
        void resetInitialSP();
        void createHumanoidRobot(const std::string& inPath);

        ChppGikStandingRobot* attStandingRobot;

        CjrlHumanoidDynamicRobot* attRobot;
        
        ChppLosSolverBasic* attSolver;

        double attSamplingPeriod;
    
        ChppLosCentipede* attCentipede;
    
        std::list<ChppLosLink*> attCentipedeParts;
    
        std::list<ChppLosTask*> attCentipedeConstraints;
        std::vector<ChppLosTask*> attTasks,attConstraints;
    
        vectorN attMask;
    
        unsigned int attNSteps;

        ChppLosTransformationTask* footT;
        ChppLosComTask* comT;
        ChppRobotMotion* attMotion;
        matrix4d tempM4,tempInv;
        void zmpInWaist(const vector3d& inZMPworPla, const vector3d& inZMPworObs, vector3d& outZMPwstObs, vector3d& outZMPwstPla);
};

#endif

